// ---------------------------------------------------------------------
// Time-stamp: <Dienstag, 08.01.2002, 10:23:20; edited by heiko>
// 
// Tina's random number generators TRNG
//
// profiling Tina's random number generators
//
// Copyright (C) 2001, 2002 Heiko Bauke
//
// heiko.bauke@physik.uni-magdeburg.de
//
// TRNG is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation. This program
// is distributed WITHOUT ANY WARRANTY; without even the implied
// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
// See the GNU General Public License for more details.
//
// ---------------------------------------------------------------------


#include <cstdlib>
#include <iostream>
#include <iomanip>
#include <vector>
#include <ctime>
#include <cmath>
#include "trng.h"
#include "fibonacci.h"

using namespace TRNG;
using namespace std;


template<class T>
void profl(TRNG::RNG<T> &r, long j) {
  clock_t my_clock;
  long i;
  cout.setf(ios::fixed);
  long sum=0;
  my_clock=clock();
  for (i=0l; i<j; ++i)
    sum+=r.rand();
  my_clock=clock()-my_clock;
#ifdef LaTeX
  cout << " & ";
#else
  cout << '\t';
#endif
  cout << setfill(' ') << setprecision(2) 
       << static_cast<double>(my_clock)/static_cast<double>(CLOCKS_PER_SEC);
  my_clock=clock();
}


template<class T>
void profd(RNG<T> &r, long j) {
  clock_t my_clock;
  long i;
  cout.setf(ios::fixed);
  double sum=0.0;
  my_clock=clock();
  for (i=0l; i<j; ++i)
    sum+=r.uniform();
  my_clock=clock()-my_clock;
#ifdef LaTeX
  cout << " & ";
#else
  cout << '\t';
#endif
  cout << setfill(' ') << setprecision(2) 
       << static_cast<double>(my_clock)/static_cast<double>(CLOCKS_PER_SEC);
}


template<class T>
void prof_main(RNG<T> &r,long from, long to) {
  long i;
  cout << r.name();
#ifdef LaTeX
  cout << " & long";
#else
  cout << "\tlong";
#endif
  for (i=(1l<<from); i<=(1l<<to); i*=4l)
    profl(r, i);
#ifdef LaTeX
  cout << "\\\\" << endl;
  cout << " & double ";
#else
  cout << endl << "\tdouble ";
#endif
  for (i=(1l<<from); i<=(1l<<to); i*=4l)
    profd(r, i);
#ifdef LaTeX
  cout << "\\\\" << endl;
#else
  cout << endl;
#endif
}


int main(void) {
  const long from=16;
  const long to=24;
  long i;
#ifdef LaTeX
  cout << " & ";
#else
  cout << "\t";
#endif
  for (i=from; i<=to; i+=2l)
#ifdef LaTeX
    cout << " & $2^{" << i << "}$";
#else
    cout << "\t2^" << i;
#endif
#ifdef LaTeX
  cout << "\\\\" << endl;
#else
  cout << endl;
#endif
  try {
    ParkMiller r1;
    LCG32 r2;
    LCG64 r3;
    MRG2 r4;
    MRG3 r5;
    MRG4 r6;
    CLCG2 r7;
    CLCG3 r8;
    CLCG4 r9;
    EINV r10;
    EINVLCG64 r11;
    Fibonacci<103, 250> r12;
    prof_main(r1, from, to);
    prof_main(r2, from, to);
    prof_main(r3, from, to);
    prof_main(r4, from, to);
    prof_main(r5, from, to);
    prof_main(r6, from, to);
    prof_main(r7, from, to);
    prof_main(r8, from, to);
    prof_main(r9, from, to);
    prof_main(r10, from, to);
    prof_main(r11, from, to);
    prof_main(r12, from, to);
  } 
  catch (error err) {
    cerr << err.message << endl;
  }
  catch (...) {
    cerr << "something went wrong" << endl;
  }
  return EXIT_SUCCESS;
}



